/*
Copyright (c) 2006-2007, Tom Thielicke IT Solutions

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301, USA.
*/

/****************************************************************
**
** Implementation of the MainWindow class
** File name: mainwindow.cpp
**
****************************************************************/

#include <QIcon>
#include <QMessageBox>
#include <QMenuBar>
#include <QSize>
#include <QPoint>
#include <QSqlQuery>
#include <QSettings>
#include <QProcess>
#include <QDir>
#include <QRect>
#include <QCoreApplication>
#include <QByteArray>
#include <QDesktopServices>

#include "mainwindow.h"
#include "settingsdialog.h"
#include "updatedialog.h"
#include "def/defines.h"
#include "def/errordefines.h"
#include "errormessage.h"

MainWindow::MainWindow() {
	trainingStarted = false;

	createActions();
	createMenu();

	createWidgets();

	createStart();
	startWidget->fillLessonList(false);
	selectedLesson = -1;
	selectedType = -1;

	setWindowTitle(APP_NAME);
	setWindowIcon(QIcon(":/img/" + QString(ICON_FILENAME)));
    setMinimumSize(APP_WIDTH_STANDARD, APP_HEIGHT_STANDARD);

    readSettings();
}

MainWindow::~MainWindow() {
	writeSettings();
}

void MainWindow::closeEvent(QCloseEvent *event) {

	if (trainingStarted) {

		// Ask, if training is already started
		switch (QMessageBox::question(this, APP_NAME,
			tr("Es gehen alle Werte der laufenden Lektion verloren!"
			"\n\nWollen Sie die Anwendung wirklich beenden?\n\n"),
			tr("&Ja"), tr("&Abbrechen"), 0, 1)) {

			case 0:
				// Ok button pushed
				break;
			case 1:
				// Cancel button pushed
				event->ignore();
				break;
		}
	}
}

bool MainWindow::checkLicenseKey(QString licenseKey) {

	return false;
}

void MainWindow::createMenu() {
	//Mac-Version:
	//-----------
	#if APP_MAC
	evaluationMenu = menuBar()->addMenu(tr("&Gehe zu"));
	evaluationMenu->addAction(exitAction);
	evaluationMenu->addAction(settingsAction);
	evaluationMenu->addAction(updateAction);
	evaluationMenu->addAction(evalAction);
	helpMenu = menuBar()->addMenu(tr("&Hilfe"));
	helpMenu->addAction(helpAction);
	helpMenu->addAction(websiteAction);
	helpMenu->addSeparator();
	helpMenu->addAction(aboutAction);
	#else
	// Win/X11-Version:
	// ---------------
	// Menu bar items
	#if APP_COMAK
	fileMenu = menuBar()->addMenu(tr("Datei"));
	#else
	fileMenu = menuBar()->addMenu(APP_NAME_INTERN);
	#endif
	fileMenu->addAction(settingsAction);
	fileMenu->addAction(updateAction);
	fileMenu->addSeparator();
	fileMenu->addAction(exitAction);
	evaluationMenu = menuBar()->addMenu(tr("&Gehe zu"));
	evaluationMenu->addAction(evalAction);
	//evaluationMenu->addSeparator();
	helpMenu = menuBar()->addMenu(tr("&Hilfe"));
	helpMenu->addAction(helpAction);
	helpMenu->addAction(websiteAction);
	helpMenu->addSeparator();
	helpMenu->addAction(aboutAction);
	#endif
}

void MainWindow::createActions() {
	// Mac-Version:
	// -----------
	#if APP_MAC
	settingsAction = new QAction(QIcon(":/img/menu_settings.png"),
		tr("&Einstellungen"), this);
	updateAction = new QAction(QIcon(":/img/menu_update.png"),
		tr("Aktualisierung"), this);
	exitAction = new QAction(tr("&Beenden"), this);
	evalAction = new QAction(QIcon(":/img/menu_evaluation.png"),
		tr("&Auswertung"), this);
	helpAction = new QAction(QIcon(":/img/menu_help.png"),
		tr("&Bedienungsanleitung"), this);
	websiteAction = new QAction(QIcon(":/img/menu_website.png"),
		APP_NAME_INTERN + tr(" im Internet"), this);
	aboutAction = new QAction(tr("&ber ") + APP_NAME_INTERN, this);
	#else
	// Win/X11-Version:
	// ---------------
	// Menu bar actions
	settingsAction = new QAction(QIcon(":/img/menu_settings.png"),
		tr("&Grundeinstellungen"), this);
	updateAction = new QAction(QIcon(":/img/menu_update.png"),
		tr("Aktualisierung"), this);
	exitAction = new QAction(tr("&Beenden"), this);
	evalAction = new QAction(QIcon(":/img/menu_evaluation.png"),
		tr("&Auswertung"), this);
	helpAction = new QAction(QIcon(":/img/menu_help.png"),
		tr("&Bedienungsanleitung"), this);
	websiteAction = new QAction(QIcon(":/img/menu_website.png"),
		APP_NAME_INTERN + tr(" im Internet"), this);
	aboutAction = new QAction(tr("&ber ") + APP_NAME_INTERN, this);
	#endif

	// Connect bar actions
	connect(aboutAction, SIGNAL(triggered()), this, SLOT(about()));
	connect(exitAction, SIGNAL(triggered()), this, SLOT(close()));
	connect(settingsAction, SIGNAL(triggered()), this, SLOT(showSettings()));
	connect(updateAction, SIGNAL(triggered()), this, SLOT(showUpdate()));
	connect(evalAction, SIGNAL(triggered()), this,
		SLOT(toggleStartToEvaluation()));
	connect(websiteAction, SIGNAL(triggered()), this, SLOT(openWebsite()));
	connect(helpAction, SIGNAL(triggered()), this, SLOT(showHelp()));
}

void MainWindow::showSettings() {
	SettingsDialog settingsDialog(this);
	settingsDialog.exec();
	// Fill lesson list after changing program settings
	startWidget->fillLessonList(false);
	startWidget->readSettings();
}

void MainWindow::showUpdate() {
	UpdateDialog updateDialog(this);
	updateDialog.exec();
	// Fill lesson list after online update
	startWidget->fillLessonList(false);
}

void MainWindow::showHelp() {
	// Open help html file in the windows standard browser
	/*QProcess p;
	QString path = QDir::currentPath() + "/help/help.html";
    QStringList s;
    s << "url.dll,FileProtocolHandler" << path;
    p.startDetached(QString("rundll32.exe") , s );*/
    helpBrowser = new HelpBrowser("", 0);
	helpBrowser->show();
}

void MainWindow::openWebsite() {
	QDesktopServices::openUrl(QString(APP_URL));
}

void MainWindow::about() {
	QSqlQuery query;
	QString dbVersion = "?";
	QString softwareVersion = "?";
	QString infoText = "";

	// Get database version info
	if (!query.exec("SELECT * FROM db_version ORDER BY version DESC;")) {
		ErrorMessage *errorMessage = new ErrorMessage(this);
		errorMessage->showMessage(ERR_DB_VERSION_READABLE,
			TYPE_WARNING, CANCEL_NO);
		// return is not necessary here - "about message" anyway
	} else {
		if (query.first()) {
			// One number represents the DB version
			dbVersion = query.value(0).toString();
			softwareVersion = query.value(1).toString();
		}
	}
	//Show about window with image
	infoText = tr("Programmversion ") + softwareVersion +
		"\n" + tr("Datenbankversion ") + dbVersion;
	if (APP_PORTABLE) {
		infoText.append("\n" + tr("Portable Version"));
	}
	IllustrationDialog illustrationDialog("about.png", 540, 400,
		QString(tr("ber ") + APP_NAME), infoText, "", 1, this);
	illustrationDialog.exec();

}

void MainWindow::createWidgets() {
	stackedWidget = new QStackedWidget(this);
	setCentralWidget(stackedWidget);
	startWidget = new StartWidget(this);

}

void MainWindow::createStart() {
	menuBar()->show();
    //setCentralWidget(startWidget);
    stackedWidget->addWidget(startWidget);
    stackedWidget->setCurrentWidget(startWidget);
	connect(startWidget, SIGNAL(trainingClicked(int, int, QString)),
		this, SLOT(toggleStartToTraining(int, int, QString)));
	if (!isMaximized() && height() < APP_HEIGHT_STANDARD) {
		resize(APP_WIDTH_STANDARD, APP_HEIGHT_STANDARD);
	}
	setMinimumSize(APP_WIDTH_STANDARD, APP_HEIGHT_STANDARD);
}

void MainWindow::deleteStart() {
    stackedWidget->removeWidget(startWidget);
	disconnect(startWidget, SIGNAL(trainingClicked(int, int, QString)),
		this, SLOT(toggleStartToTraining(int, int, QString)));
	startWidget->writeSettings();
	//delete startWidget;
}

void MainWindow::createTraining(int lesson, int type, QString name) {
	trainingStarted = true;
	menuBar()->hide();
	trainingWidget = new TrainingWidget(lesson, type, name, this);
	//setCentralWidget(trainingWidget);
    stackedWidget->addWidget(trainingWidget);
    stackedWidget->setCurrentWidget(trainingWidget);
    trainingWidget->tickerBoard->setFocus();
    connect(trainingWidget, SIGNAL(lessonReady(int, int, QList<QChar>, QList<int>)),
		this, SLOT(toggleTrainingToEvaluation(int, int, QList<QChar>, QList<int>)));
	connect(trainingWidget, SIGNAL(lessonCanceled()),
		this, SLOT(toggleTrainingToStart()));
}

void MainWindow::deleteTraining() {
    stackedWidget->removeWidget(trainingWidget);
	trainingStarted = false;
	disconnect(trainingWidget, SIGNAL(lessonReady(int, int, QList<QChar>, QList<int>)),
		this, SLOT(toggleTrainingToEvaluation(int, int, QList<QChar>, QList<int>)));
	disconnect(trainingWidget, SIGNAL(lessonCanceled()),
		this, SLOT(toggleTrainingToStart()));
	delete trainingWidget;
}

void MainWindow::createEvaluation(int row, int type, QList<QChar> charList,
	QList<int> mistakeList) {

	menuBar()->hide();
	evaluationWidget = new EvaluationWidget(row, type, charList, mistakeList, this);
    //setCentralWidget(evaluationWidget);
    stackedWidget->addWidget(evaluationWidget);
    stackedWidget->setCurrentWidget(evaluationWidget);
	connect(evaluationWidget, SIGNAL(readyClicked()),
		this, SLOT(toggleEvaluationToStart()));
	if (!isMaximized() && height() < APP_HEIGHT_STANDARD) {
		resize(APP_WIDTH_STANDARD, APP_HEIGHT_STANDARD);
	}
	setMinimumSize(APP_WIDTH_STANDARD, APP_HEIGHT_STANDARD);
}

void MainWindow::deleteEvaluation() {
    stackedWidget->removeWidget(evaluationWidget);
	disconnect(evaluationWidget, SIGNAL(readyClicked()),
		this, SLOT(toggleEvaluationToStart()));
	delete evaluationWidget;
}

void MainWindow::toggleStartToTraining(int lesson, int type, QString name) {
	selectedLesson = lesson;
	selectedType = type;
	selectedName = name;
	deleteStart();
	startWidget->fillLessonList(false);
	createTraining(lesson, type, name);
}

void MainWindow::toggleTrainingToStart() {
	deleteTraining();
	createStart();
}

void MainWindow::toggleTrainingToEvaluation(int row, int type, QList<QChar> charList,
	QList<int> mistakeList) {

	deleteTraining();
	createEvaluation(row, type, charList, mistakeList);
}

void MainWindow::toggleEvaluationToStart() {
	deleteEvaluation();
	createStart();
	startWidget->fillLessonList(false);
}

void MainWindow::toggleStartToEvaluation() {
	deleteStart();
	QList<QChar> charList;
	QList<int> mistakeList;
	createEvaluation(0, 0, charList, mistakeList);
}

void MainWindow::readSettings() {
	#if APP_PORTABLE
	QSettings settings(QSettings::IniFormat, QSettings::UserScope, "portable", "settings");
    settings.setPath(QSettings::IniFormat, QSettings::UserScope, QCoreApplication::applicationDirPath());
	#else
	QSettings settings;
	#endif

	settings.beginGroup("mainwindow");
	#if APP_WIN
	QByteArray storedGeometry;
	storedGeometry = settings.value("geometry").toByteArray();
	if (storedGeometry.isEmpty() || storedGeometry.isNull()) {
		resize(QSize(APP_WIDTH_STANDARD, APP_HEIGHT_STANDARD));
		move(QPoint(100, 100));
	} else {
		restoreGeometry(storedGeometry);
	}
	#else
	resize(settings.value("size", QSize(APP_WIDTH_STANDARD, APP_HEIGHT_STANDARD)).toSize());
	move(settings.value("pos", QPoint(100, 100)).toPoint());
	#endif
	settings.endGroup();
}

void MainWindow::writeSettings() {
	#if APP_PORTABLE
	QSettings settings(QSettings::IniFormat, QSettings::UserScope, "portable", "settings");
    settings.setPath(QSettings::IniFormat, QSettings::UserScope, QCoreApplication::applicationDirPath());
	#else
	QSettings settings;
	#endif

    settings.beginGroup("mainwindow");
	#if APP_WIN
	settings.setValue("geometry", saveGeometry());
	#else
	settings.setValue("size", size());
	settings.setValue("pos", pos());
	#endif
	settings.endGroup();
}
